from threading import Thread
from dataclasses import dataclass
import time

from .messages import printf

@dataclass
class Spinners:
    line_spinner = [r"|", "\\", "-", "/"]
    mark_spinner = ["#", "!", "?", "$", "&"]

class loadbar(object):
    def __init__(self, spinner: list[str], __str: str, speed=0.4, spinner_color = None) -> None:
        self.spinner = spinner
        self.text = __str
        self.is_stopped = False
        self.speed = speed
        self.spinner_color = spinner_color

    def spin(self):
        while not self.is_stopped: 
            for a in self.spinner:
                if self.spinner_color:
                    a = self.spinner_color + a + "\033[0m"

                printf("\r" + a, self.text, end="", flush=True)
                time.sleep(self.speed)

    def __enter__(self):
        Thread(target=self.spin).start()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.is_stopped = True



from setuptools import find_packages, setup

setup(
    name='stone_color',
    packages=find_packages(),
    version='0.3.0',
    long_description='Stone_color is a style library for Python that can perform text formatting, coloring, tables, and loading bars.',
    description='Text formatting, coloring and tables library',
    keywords=["color", "cli", "table", "terminal-colors", "ansi", "rgb"],
    author='aniko33',
)



import io

def ascii_table(header: list[str], data: list[list[str]], spaces=4, lspace=2, start_end="") -> str:
    table = io.StringIO()
    
    div = " "*spaces
    ldiv = " "*lspace

    col_widths = [max(len(str(cell)) for cell in col) for col in zip(header, *data)]

    header_row = div.join(f"{header: <{width}}" for header, width in zip(header, col_widths))
    table.write(start_end + ldiv + header_row + "\n")

    separator_row = div.join("-" * width for width in col_widths)
    table.write(ldiv + separator_row + "\n")

    for row in data:
        row_str = div.join(f"{cell: <{width}}" for cell, width in zip(row, col_widths))
        table.write(ldiv + row_str + "\n")

    table.write(start_end)

    return table.getvalue()

def unicode_light_table(header: list[str], data: list[list[str]], lspace=0, start_end="") -> str:
    table = io.StringIO()

    ldiv = " "*lspace
    col_widths = [max(len(str(item)) for item in col) for col in zip(*data, header)]

    table.write(start_end + ldiv + "┌" + "─" * (sum(col_widths) + len(col_widths) * 3 - 1) + "┐\n")
    table.write(ldiv + "│")
    for header_item, width in zip(header, col_widths):
        table.write(f" {header_item.ljust(width)} │")
    table.write("\n" + ldiv + "├" + "─" * (sum(col_widths) + len(col_widths) * 3 - 1) + "┤\n")

    for row in data:
        table.write(ldiv + "│")
        for item, width in zip(row, col_widths):
            table.write(f" {item.ljust(width)} │")
        table.write("\n")

    table.write(ldiv + "└" + "─" * (sum(col_widths) + len(col_widths) * 3 - 1) + "┘" + start_end)

    return table.getvalue()


def unicode_heavy_table(header: list[str], data: list[list[str]], lspace=0, start_end="") -> str:
    table = io.StringIO()

    ldiv = " "*lspace
    col_widths = [max(len(str(item)) for item in col) for col in zip(*data, header)]

    table.write(start_end + ldiv + "╔" + "═" * (sum(col_widths) + len(col_widths) * 3 - 1) + "╗\n")
    table.write(ldiv + "║")
    for header_item, width in zip(header, col_widths):
        table.write(f" {header_item.ljust(width)} ║")
    table.write("\n" + ldiv + "╠" + "═" * (sum(col_widths) + len(col_widths) * 3 - 1) + "╣\n")

    for row in data:
        table.write(ldiv + "║")
        for item, width in zip(row, col_widths):
            table.write(f" {item.ljust(width)} ║")
        table.write("\n")

    table.write(ldiv + "╚" + "═" * (sum(col_widths) + len(col_widths) * 3 - 1) + "╝" + start_end)

    return table.getvalue()



from sys import stdout, stderr
import re

from . import color

colors = color.DefaultColors

_alert = colors.highred + "[!]" + colors.reset
_warn = colors.highyellow + "[@]" + colors.reset
_info = colors.cyan + "[?]" + colors.reset 
_success = colors.green + "[+]" + colors.reset
_error = colors.red + "[-]" + colors.reset 

ansi_style = {
    "bold": "\033[1m",
    "italic": "\033[3m",
    "underline": "\033[4m",
    "blink": "\033[5m",
    "rapidblink": "\033[6m",
    "strike": "\033[9m",
    "invert": "\033[7m",
    "reset": "\033[0m"
}

auto_reset = False

def alertf(*objs, end="\n", sep=" ", file=stderr, flush=False):
    printf(_alert, *objs, end=end, sep=sep, file=file, flush=flush)

def warnf(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    printf(start + _warn, *objs, end=end, sep=sep, file=file, flush=flush)

def infof(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    printf(start + _info, *objs, end=end, sep=sep, file=file, flush=flush)

def successf(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    printf(start + _success, *objs, end=end, sep=sep, file=file, flush=flush)
    
def errorf(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    printf(start + _error, *objs, end=end, sep=sep, file=file, flush=flush)

def printf(*objs, end="\n", sep=" ", file=stdout, flush=False):
    __text = sep.join(map(str, objs)) + end

    __text = formatf(__text)
    
    if auto_reset:
        __text += "\033[0m"

    file.write(__text) 
    
    if flush:
        file.flush()
    
def formatf(*objs, sep=" ") -> str:
    __text = sep.join(map(str, objs))

    def color_replace(match):
        hex = match.group(1)

        if hex in ansi_style.keys():
            return ansi_style[hex]
        else:
            return color.chex(hex)

    __text = re.sub(r"\{#([^}]+)\}", color_replace, __text)

    return __text
